#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_imcagui_gui { record_list } {

	create_imcagui_menubar $record_list

	create_imcagui_main_window $record_list

	setup_imcagui_callbacks $record_list
}

proc setup_imcagui_callbacks { record_list } {

	find_callback_record $record_list "lambda"
	find_callback_record $record_list "d_spacing"
	find_callback_record $record_list "bh_center"
	find_callback_record $record_list "bv_center"
	find_callback_record $record_list "bh_size"
	find_callback_record $record_list "bv_size"
	find_callback_record $record_list "edge_energy"

	if { $IMCAGui::program_name == "imca17id" } {
		find_callback_record $record_list "id_ev"
		find_callback_record $record_list "id_ev_harmonic"
		find_callback_record $record_list "id_ev_params"
		find_callback_record $record_list "id_ev_enabled"
		find_callback_record $record_list "filter1"
		find_callback_record $record_list "filter2"
	}

	after 1 [ list process_imcagui_callbacks ]
}

proc find_callback_record { record_list record_name } {

	global callback_records

	set status [ catch { $record_list get_record $record_name } \
			record ]

	if { $status != 0 } {
		log_puts \
		"Error: '$record_name' record not found in database."

		return -code error
	}

	set callback_records($record_name) $record
}

proc process_imcagui_callbacks { } {

	global callback_list
	global callback_records
	global callback_values

	if [ catch { $callback_records(lambda) get_position } \
					callback_values(lambda) ] \
	{
		set callback_values(lambda) "*****"
	}

	set callback_values(energy) \
		[ energy_from_lambda $callback_values(lambda) ]

	if [ catch { $callback_records(d_spacing) receive_variable } \
					callback_values(d_spacing) ] \
	{
		set callback_values(d_spacing) "*****"
	}

	if [ catch { $callback_records(bh_center) get_position } \
					callback_values(bh_center) ] \
	{
		set callback_values(bh_center) "*****"
	}

	if [ catch { $callback_records(bv_center) get_position } \
					callback_values(bv_center) ] \
	{
		set callback_values(bv_center) "*****"
	}

	if [ catch { $callback_records(bh_size) get_position } \
					callback_values(bh_size) ] \
	{
		set callback_values(bh_size) "*****"
	}

	if [ catch { $callback_records(bv_size) get_position } \
					callback_values(bv_size) ] \
	{
		set callback_values(bv_size) "*****"
	}

	if { $IMCAGui::program_name == "imca17id" } {
		if [ catch { $callback_records(id_ev) get_position } \
					callback_values(id_ev) ] \
		{
			set callback_values(id_ev) "*****"
		}

		if [ catch { $callback_records(id_ev_harmonic) \
					receive_variable } \
					callback_values(id_ev_harmonic) ] \
		{
			set callback_values(id_ev_harmonic) "*****"
		}

		if [ catch { $callback_records(filter1) get_position } \
					callback_values(filter1) ] \
		{
			set callback_values(filter1) "*****"
		}

		if [ catch { $callback_records(filter2) get_position } \
					callback_values(filter2) ] \
		{
			set callback_values(filter2) "*****"
		}
	}

	foreach callback $callback_list {
		$callback
	}

	after 1000 [ list process_imcagui_callbacks ]
}

proc add_imcagui_callback { callback } {
	global callback_list

	lappend callback_list $callback
}

proc delete_imcagui_callback { callback } {
	global callback_list

	set i [ lsearch -exact $callback_list $callback ]

	if { $i < 0 } {
		set t \
	    "Attempt to delete callback '$callback' from callback list failed."

		tk_dialog .delete_callback_failed \
		"Callback Deletion Failed." $t error 0 "OK"

		return
	}

	set callback_list [ lreplace $callback_list $i $i ]
}

proc create_imcagui_menubar { record_list } {
	menu .menubar
	. configure -menu .menubar

	###############################################################
	#
	#   Add the File Menu
	#

	menu .menubar.file -tearoff 0
	.menubar add cascade -label File -menu .menubar.file

	.menubar.file add command -label Print -command [ code print_window . ]

	.menubar.file add command -label Staff \
		-command [ code eval exec "mxgui &" ]

	.menubar.file add command -label Exit -command exit

	###############################################################
	#
	#   Add the Edit Menu
	#

	menu .menubar.edit -tearoff 0
	.menubar add cascade -label Edit -menu .menubar.edit

	.menubar.edit add command -label "Set Default Printer" \
				-command set_default_printer

	###############################################################
	#
	#   Add the Variables Menu
	#

	menu .menubar.variables -tearoff 0
	.menubar add cascade -label Variables -menu .menubar.variables

	.menubar.variables add command -label "Edge Energy" \
				-command set_energy

	if { $IMCAGui::program_name == "imca17id" } {
		.menubar.variables add command -label "ID_ev_enabled" \
				-command set_ev_enabled

		.menubar.variables add command -label "ID_ev_harmonic" \
				-command set_harmonic

		.menubar.variables add command -label "ID_ev_params" \
				-command set_ev_params
	}
	###############################################################
	#
	#   Add the Spectroscopy / MAD Menu
	#

	menu .menubar.mad -tearoff 0
	.menubar add cascade -label Experiment -menu .menubar.mad

	.menubar.mad add command -label Spectroscopy -command [ code \
		create_experiment_window Spectroscopy $record_list ]

	.menubar.mad add command -label MAD -command [ code \
		create_experiment_window MAD $record_list ]

	###############################################################
	#
	#   Add the Help Menu
	#

	menu .menubar.help -tearoff 0
	.menubar add cascade -label Help -menu .menubar.help

	.menubar.help add command -label "About IMCA GUI" \
			-command menubar_about_command

	.menubar.help add command -label "Help for IMCA 17-ID" \
			-command [ code display_help "imca17id.html" ]

	.menubar.help add command -label "Help for IMCA 17-BM" \
			-command [ code display_help "imca17bm.html" ]

	.menubar.help add command -label "IMCA FAQ" \
			-command [ code display_help "imca_faq.html" ]

	.menubar.help add command -label "IMCA-CAT FAQ" \
			-command [ code display_help "imca_cat_faq.html" ]

	proc menubar_about_command {} {
		set t "IMCA GUI was written by William Lavender based on "
		append t "a design by the IMCA-CAT staff."

		tk_dialog .about_dialog "About IMCA GUI" $t info 0 "Done"
	}
}

#
#  Create the main window.
#
proc create_imcagui_main_window { record_list } {

	set window .main

	frame .main
	pack .main -before .message_log

	#-----

	set frame .main.energy_frame

	frame $frame
	pack $frame -side top

	frame $frame.grid
	pack $frame.grid -side left

	create_imcagui_energy_frame $record_list $frame.grid

	frame $frame.sep
	pack $frame.sep -side left -padx 5

	button $frame.move_button -text Move -command \
			[ code move_lambda $frame.grid ]

	pack $frame.move_button -side left

	#-----

	frame .main.sep1 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep1 -expand yes -fill x -pady 5 -side top

	#-----

	set frame .main.harmonic_frame

	frame $frame
	pack $frame -side top

	create_harmonic_frame $frame

	#-----

	frame .main.sep2 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep2 -expand yes -fill x -pady 5 -side top

	#-----

	set slit_frame .main.slit_frame

	frame $slit_frame
	pack $slit_frame -side top

	create_slit_frame $slit_frame

	if { $IMCAGui::program_name == "imca17id" } {
		#-----

		frame .main.sep3 -height 2 -borderwidth 1 -relief sunken
		pack .main.sep3 -expand yes -fill x -pady 5 -side top

		#-----

	        set frame .main.filter_frame

	        frame $frame
	        pack $frame -side top
#		button $frame.filters -text "Attenuator Control..." \
#			-command [ code create_filter_window $record_list ]

#		pack $frame.filters -side left
		create_filter_frame $frame $record_list
		frame $frame.sep
		pack $frame.sep -side left -padx 5
	}

	#-----

	frame .main.sep5 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep5 -expand yes -fill x -pady 5 -side top

	set frame .main.bottom

	frame $frame
	pack $frame -side top
	set command "optimize_gui &"

	button $frame.optimize -text "Optimize Intensity..." \
		-command [ code eval exec $command ]

	pack $frame.optimize -side left

	#-----

	frame .main.sep4 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep4 -expand yes -fill x -pady 5 -side top

	#-----

	add_imcagui_callback main_window_callback
}

proc main_window_callback { } {

	global callback_values

	set window .main

	set frame $window.energy_frame.grid

	$frame.energy_value configure -text \
		[ imcagui_format "%10.3f" $callback_values(energy) ]

	$frame.lambda_value configure -text \
		[ imcagui_format "%8.4f"  $callback_values(lambda) ]

	if { $IMCAGui::program_name == "imca17id" } {

		$frame.id_value configure -text \
			[ imcagui_format "%10.3f" $callback_values(id_ev) ]
	}

	set frame $window.harmonic_frame

	if { $IMCAGui::program_name == "imca17id" } {

		$frame.harmonic_value configure -text \
		    [ imcagui_format "%2d" $callback_values(id_ev_harmonic) ]
	}

	$frame.d_spacing_value configure -text \
		[ imcagui_format "%8.5f" $callback_values(d_spacing) ]

	set frame $window.slit_frame

	$frame.horiz_size_value configure -text \
		[ imcagui_format "%10.3f" $callback_values(bh_size) ]

	$frame.horiz_center_value configure -text \
		[ imcagui_format "%10.3f" $callback_values(bh_center) ]

	$frame.vert_size_value configure -text \
		[ imcagui_format "%10.3f" $callback_values(bv_size) ]

	$frame.vert_center_value configure -text \
		[ imcagui_format "%10.3f" $callback_values(bv_center) ]

	if { $IMCAGui::program_name == "imca17id" } {
        	set window .main.filter_frame.grid.filter_current
        	set thickness [ get_filter_thickness ]

 	        $window configure -text $thickness

        	if { $thickness == "Fault" } {
                	$window configure -fg red
        	} elseif { $thickness == "Moving" } {
                	$window configure -fg limegreen
        	} elseif { $thickness == "0.0" } {
                	$window configure -fg blue
        	} else {
                	$window configure -fg red
        	}
        }

}

proc imcagui_format { format_specifier value } {

	if [ catch { format $format_specifier $value } result ] {
		set result $value
	}
	return $result
}

############################################################################
#
#  Create the energy and wavelength display frame.
#
proc create_imcagui_energy_frame { record_list frame } {

	set row 0

	label $frame.current_size_label -text Current -width 10
	label $frame.desired_size_label -text Desired 

	grid $frame.current_size_label -row $row -column 1
	grid $frame.desired_size_label -row $row -column 3

	incr row

	label $frame.energy_label -text Energy
	label $frame.energy_value -text "---" -fg blue
	label $frame.energy_units1 -text "eV   "
	label $frame.energy_units2 -text eV

	iwidgets::entryfield $frame.energy_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P } \
		-command [ code energy_desired_update $frame ]

	grid $frame.energy_label   -row $row -column 0 -sticky w
	grid $frame.energy_value   -row $row -column 1 -sticky e
	grid $frame.energy_units1  -row $row -column 2 -sticky w
	grid $frame.energy_desired -row $row -column 3
	grid $frame.energy_units2  -row $row -column 4 -sticky w

	incr row

	label $frame.lambda_label -text Wavelength
	label $frame.lambda_value -text "---" -fg blue
	label $frame.lambda_units1 -text A
	label $frame.lambda_units2 -text A

	iwidgets::entryfield $frame.lambda_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P } \
		-command [ code lambda_desired_update $frame ]

	grid $frame.lambda_label   -row $row -column 0 -sticky w
	grid $frame.lambda_value   -row $row -column 1 -sticky e
	grid $frame.lambda_units1  -row $row -column 2 -sticky w
	grid $frame.lambda_desired -row $row -column 3
	grid $frame.lambda_units2  -row $row -column 4 -sticky w

	if { $IMCAGui::program_name == "imca17id" } {
		incr row

		label $frame.id_label -text "Gap Energy"
		label $frame.id_value -text "---" -fg blue
		label $frame.id_units -text eV

		grid $frame.id_label -row $row -column 0 -sticky w
		grid $frame.id_value -row $row -column 1 -sticky e
		grid $frame.id_units -row $row -column 2 -sticky w
	}

	bind $frame.energy_desired.lwchildsite.entry <KeyRelease> \
		[ code energy_desired_update $frame ]

	bind $frame.lambda_desired.lwchildsite.entry <KeyRelease> \
		[ code lambda_desired_update $frame ]
}

proc energy_desired_update { frame } {

	set energy [ $frame.energy_desired get ]

	set lambda [ lambda_from_energy $energy ]

	$frame.lambda_desired clear
	$frame.lambda_desired insert end $lambda
}

proc lambda_desired_update { frame } {

	set lambda [ $frame.lambda_desired get ]

	set energy [ energy_from_lambda $lambda ]

	$frame.energy_desired clear
	$frame.energy_desired insert end $energy
}

proc move_lambda { frame } {

	global callback_records
	global callback_values

	set lambda_requested [ $frame.lambda_desired get ]

	if { $lambda_requested == "" } {

		set energy_requested [ $frame.energy_desired get ]

		if { $energy_requested == "" } {
			set t "No energy or wavelength value requested."

			tk_dialog .no_wavelength_dialog \
				"No Value Specified" $t error 0 "OK"

			return
		}

		set lambda_requested \
			[ lambda_from_energy $energy_requested ]
	}

	set status [ catch { expr $lambda_requested } junk ]

	if { $status != 0 } {
		set t "Invalid wavelength value '$lambda_requested' requested."

		tk_dialog .invalid_wavelength_dialog \
			"Invalid Wavelength" $t error 0 "OK"

		return
	}

#	if [ expr { $lambda_requested > 1.16 } ] {
#		$callback_records(id_ev_harmonic) send_variable 1
#	} else {
#		$callback_records(id_ev_harmonic) send_variable 3
#	}

	$callback_records(lambda) move_absolute $lambda_requested
}

############################################################################

proc create_harmonic_frame { frame } {

	if { $IMCAGui::program_name == "imca17id" } {
		label $frame.harmonic_label -text "Undulator\nHarmonic" 
		pack $frame.harmonic_label -side left

		frame $frame.sep1 -width 1 -borderwidth 1
		pack $frame.sep1 -side left -expand yes -fill y -padx 5

		label $frame.harmonic_value -text "-" -fg blue
		pack $frame.harmonic_value -side left

		frame $frame.sep2 -width 1 -borderwidth 1
		pack $frame.sep2 -side left -expand yes -fill y -padx 20
	}

	label $frame.d_spacing_label -text "Monochromator\nd Spacing"
	pack $frame.d_spacing_label -side left

	frame $frame.sep3 -width 1 -borderwidth 1
	pack $frame.sep3 -side left -expand yes -fill y -padx 5

	label $frame.d_spacing_value -text "---" -fg blue
	pack $frame.d_spacing_value -side left

	frame $frame.sep4 -width 1 -borderwidth 1
	pack $frame.sep4 -side left -expand yes -fill y -padx 5

	label $frame.d_spacing_units -text "A"
	pack $frame.d_spacing_units -side left
}

############################################################################
proc create_filter_frame { frame  record_list} {

        set grid $frame.grid

        frame $grid
        pack $grid -side top

        # Create the filter control grid top labels.

	label $grid.frame_label -text "Attenuation:  "
        label $grid.current -text "Current"
        label $grid.desired -text "Desired"

	grid $grid.frame_label  -row 0 -column 0
        grid $grid.current -row 0 -column 1 -columnspan 2
        grid $grid.desired -row 0 -column 3 -columnspan 2

        # Create the filter row.

        label $grid.filter_label -text "Al Thickness "
        label $grid.filter_current -fg blue -text "---"
        label $grid.filter_current_units -text "um  "        

	iwidgets::optionmenu $grid.filter_desired

        set sorted_thickness_list [ lsort -real $IMCAGui::thickness_list ]

        foreach item $sorted_thickness_list {
                $grid.filter_desired insert end $item
        }

#        set thickness [ get_filter_thickness ]
        set thickness 0
        if { $thickness == "Moving" } {
                set thickness 0
        }
        if { $thickness == "Fault" } {
                set thickness 0
        }
        set selection [ lsearch -exact $sorted_thickness_list $thickness ]

        if { $selection < 0 } {
                set selection 0
        }

        $grid.filter_desired select $selection

        button $grid.move -text "Move" \
                -command [ code move_filters $record_list $grid ]

        label $grid.filter_desired_units -text "um  "

        grid $grid.filter_label         -row 1 -column 0
        grid $grid.filter_current       -row 1 -column 1
        grid $grid.filter_current_units -row 1 -column 2 -sticky w
        grid $grid.filter_desired       -row 1 -column 3
        grid $grid.filter_desired_units -row 1 -column 4 -sticky w
        grid $grid.move                 -row 1 -column 5

#        frame $frame.sep6 -height 2 -borderwidth 1 -relief sunken
#        pack $frame.sep6 -expand yes -fill x -pady 5 -side top

        # Create the bottom row.

        #set bottom $frame.bottom
        set bottom $frame

        #frame $bottom
        #pack $bottom -side top

#        pack $bottom.move -side right
}
############################################################################

proc create_slit_frame { frame } {

	set row 0

	label $frame.frame_label -text "Slits:"

	label $frame.size_label -text "Slit Sizes"
	label $frame.center_label -text "Slit Positions"

	grid $frame.frame_label  -row $row -column 0
	grid $frame.size_label   -row $row -column 1 -columnspan 2
	grid $frame.center_label -row $row -column 3 -columnspan 2

	incr row

	label $frame.current_size_label -text Current -width 10
	label $frame.desired_size_label -text Desired
	label $frame.current_center_label -text Current -width 10
	label $frame.desired_center_label -text Desired

	grid $frame.current_size_label   -row $row -column 1
	grid $frame.desired_size_label   -row $row -column 2
	grid $frame.current_center_label -row $row -column 3
	grid $frame.desired_center_label -row $row -column 4

	incr row

	label $frame.vert_label -text "Vertical"
	label $frame.vert_size_value -text "---" -fg blue

	iwidgets::entryfield $frame.vert_size_desired -width 8 \
		-validate { iwidgets::Entryfield::real %P }

	label $frame.vert_center_value -text "---" -fg blue

	iwidgets::entryfield $frame.vert_center_desired -width 8 \
		-validate { iwidgets::Entryfield::real %P }

	label $frame.vert_units -text "um"

	grid $frame.vert_label          -row $row -column 0 -sticky w
	grid $frame.vert_size_value     -row $row -column 1 -sticky e
	grid $frame.vert_size_desired   -row $row -column 2
	grid $frame.vert_center_value   -row $row -column 3 -sticky e
	grid $frame.vert_center_desired -row $row -column 4
	grid $frame.vert_units          -row $row -column 5 -sticky w

	incr row

	label $frame.horiz_label -text "Horizontal"
	label $frame.horiz_size_value -text "---" -fg blue

	iwidgets::entryfield $frame.horiz_size_desired -width 8 \
		-validate { iwidgets::Entryfield::real %P }

	label $frame.horiz_center_value -text "---" -fg blue

	iwidgets::entryfield $frame.horiz_center_desired -width 8 \
		-validate { iwidgets::Entryfield::real %P }

	label $frame.horiz_units -text "um"

	grid $frame.horiz_label          -row $row -column 0 -sticky w
	grid $frame.horiz_size_value     -row $row -column 1 -sticky e
	grid $frame.horiz_size_desired   -row $row -column 2
	grid $frame.horiz_center_value   -row $row -column 3 -sticky e
	grid $frame.horiz_center_desired -row $row -column 4
	grid $frame.horiz_units          -row $row -column 5 -sticky w

	incr row

	button $frame.move_sizes -text "Change Sizes" \
			-command [ code move_slit_sizes $frame ]

	grid $frame.move_sizes -pady 5 -row $row -column 1 -columnspan 2

	button $frame.move_centers -text "Change Positions" \
			-command [ code move_slit_centers $frame ]

	grid $frame.move_centers -pady 5 -row $row -column 3 -columnspan 2
}

proc move_slit_sizes { frame } {

	global callback_records

	set horiz_requested [ $frame.horiz_size_desired get ]

	if { $horiz_requested != "" } {
		$callback_records(bh_size) move_absolute $horiz_requested
	}

	set vert_requested [ $frame.vert_size_desired get ]

	if { $vert_requested != "" } {
		$callback_records(bv_size) move_absolute $vert_requested
	}
}

proc move_slit_centers { frame } {

	global callback_records

	set horiz_requested [ $frame.horiz_center_desired get ]

	if { $horiz_requested != "" } {
		$callback_records(bh_center) move_absolute $horiz_requested
	}

	set vert_requested [ $frame.vert_center_desired get ]

	if { $vert_requested != "" } {
		$callback_records(bv_center) move_absolute $vert_requested
	}
}

proc set_energy {} {
        global callback_records

        set toplevel .energy_window

        set status [ catch { toplevel $toplevel } ]

        # If the window already exists, bring it to the top.

        if { $status != 0 } {
                raise $toplevel
                focus $toplevel
                return
        }

        wm title $toplevel "Edge Energy Control"

        set grid $toplevel.grid

        frame $grid
        pack $grid -side top
# set up entry field with current value
	iwidgets::entryfield $grid.value -labeltext "Edge energy"
	grid $grid.value     -row 0 -column 0
        window_callback edge_energy $grid.value
# set up bottom row
        set bottom $toplevel.bottom

        frame $bottom
        pack $bottom -side top

        button $bottom.move -text "Set" \
                -command [ code set_value \
                edge_energy $grid ]

        pack $bottom.move -side left

        frame $bottom.sep
        pack $bottom.sep -side left -padx 5

        button $bottom.dismiss -text "Dismiss" \
                -command [ code destroy $toplevel ]
        pack $bottom.dismiss -side left

}

proc set_ev_enabled {} {
        global callback_records

        set toplevel .ev_enable_window

        set status [ catch { toplevel $toplevel } ]

        # If the window already exists, bring it to the top.

        if { $status != 0 } {
                raise $toplevel
                focus $toplevel
                return
        }

        wm title $toplevel "ID Gap Enable Control"

        set grid $toplevel.grid

        frame $grid
        pack $grid -side top
# set up entry field with current value
	iwidgets::entryfield $grid.value -labeltext "Gap Enable"
	grid $grid.value     -row 0 -column 0
        window_callback id_ev_enabled $grid.value
# set up bottom row
        set bottom $toplevel.bottom

        frame $bottom
        pack $bottom -side top

        button $bottom.move -text "Set" \
                -command [ code set_value \
                id_ev_enabled $grid ]

        pack $bottom.move -side left

        frame $bottom.sep
        pack $bottom.sep -side left -padx 5

        button $bottom.dismiss -text "Dismiss" \
                -command [ code destroy $toplevel ]
        pack $bottom.dismiss -side left
}

proc set_ev_params {} {

        global callback_records

        set toplevel .ev_params_window

        set status [ catch { toplevel $toplevel } ]

        # If the window already exists, bring it to the top.

        if { $status != 0 } {
                raise $toplevel
                focus $toplevel
                return
        }

        wm title $toplevel "Gap Energy Offset Parameters"

        set grid $toplevel.grid

        frame $grid
        pack $grid -side top
# set up entry field with current value
	iwidgets::entryfield $grid.value -labeltext "Gap Parameters"
	grid $grid.value     -row 0 -column 0
        window_callback id_ev_params $grid.value
# set up bottom row
        set bottom $toplevel.bottom

        frame $bottom
        pack $bottom -side top

        button $bottom.move -text "Set" \
                -command [ code set_value \
                id_ev_params $grid ]

        pack $bottom.move -side left

        frame $bottom.sep
        pack $bottom.sep -side left -padx 5

        button $bottom.dismiss -text "Dismiss" \
                -command [ code destroy $toplevel ]
        pack $bottom.dismiss -side left
}

proc set_harmonic {} {
        global callback_records

        set toplevel .harmonic_window

        set status [ catch { toplevel $toplevel } ]

        # If the window already exists, bring it to the top.

        if { $status != 0 } {
                raise $toplevel
                focus $toplevel
                return
        }

        wm title $toplevel "Harmonic Control"

        set grid $toplevel.grid

        frame $grid
        pack $grid -side top
# set up entry field with current value
	iwidgets::entryfield $grid.value -labeltext "Harmonic"
	grid $grid.value     -row 0 -column 0
        window_callback id_ev_harmonic $grid.value
# set up bottom row
        set bottom $toplevel.bottom

        frame $bottom
        pack $bottom -side top

        button $bottom.move -text "Set" \
                -command [ code set_value \
                id_ev_harmonic $grid ]

        pack $bottom.move -side left

        frame $bottom.sep
        pack $bottom.sep -side left -padx 5

        button $bottom.dismiss -text "Dismiss" \
                -command [ code destroy $toplevel ]

        pack $bottom.dismiss -side left


}
proc window_callback { record window } {
        global callback_records
	global callback_values
#        set window .harmonic_window.grid.value

#        if { [ winfo exists $window ] == 0 } {
#                delete_imcagui_callback harmonic_window_callback
#                return
#        }
# get value of variable
        if [ catch { $callback_records($record) \
                                receive_variable } \
                                callback_values($record) ] \
        {
                set callback_values($record) "*****"
        }

#        { $callback_records(id_ev_harmonic) receive_variable } $value
# set edit field text
        $window clear
        $window insert end [ imcagui_format "%2d" \
			 $callback_values($record) ]

}
proc set_value { record grid } {
        global callback_records
	global callback_values
    set value_desired [ $grid.value get ]
    $callback_records($record) send_variable $value_desired
}
