/*
 * process_state.c -- A helper program for 'imcagui'.
 *
 * Usage: process_state pid
 *
 * This program determines whether or not the specified process
 * is running.  By using this small helper program, we avoid
 * the need to install TclX or BLT just to be able to know when
 * the Chooch process has exited.
 *
 * The program prints out the text string "1" if the process exists,
 * the string "0" is the process doesn't exist, or a text string
 * describing the error that occurred.
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>

int
main( int argc, char *argv[] )
{
	int status, saved_errno;
	pid_t pid;

	if ( argc != 2 ) {
		printf( "Wrong number of arguments.\n" );
		exit(0);
	}

	pid = atol( argv[1] );

	if ( pid == 0 ) {
		printf( "PID argument is not a number.\n" );
		exit(0);
	}

	/* Sending signal 0 allows one to determine whether or not
	 * a given process exists.
	 */

	status = kill( pid, 0 );

	if ( status == 0 ) {
		printf( "1\n" );
		exit(0);
	}
	if ( status != -1 ) {
		printf( "Unexpected status code %d from kill().\n", status );
		exit(0);
	}

	saved_errno = errno;

	switch( saved_errno ) {
	case ESRCH:		/* The process doesn't exist. */
		printf( "0\n" );
		break;
	case EPERM:
		printf( "Permission denied.\n" );
		break;
	case EINVAL:
		printf( "Invalid signal number.\n" );
		break;
	default:
		printf( "Unexpected errno value %d.\n", saved_errno );
		break;
	}
	exit(0);
}

